Forgery = LibStub("AceAddon-3.0"):NewAddon("Forgery", "AceConsole-3.0", "AceEvent-3.0")


local options = {
	name = "Forgery",
	handler = Forgery, 
	type = 'group', 
	desc = "Forges for you.",
	args = {

		h = {
			type = "header",
			name = "Equivalence Values",
			width = "half",
			order = 0,
		},
		test = {
			type = "execute",
			name = "test",
			func = "test",
			desc = "Tests something", 
	--		usage = "/kc test"
			order = 1,
		},
		h2 = {
			type = "header",
			name = "Equivalence Values",
			order = 2,
		},
	},
}


local _STATLIST = {
	-- Base stats
	stamina = { "ITEM_MOD_STAMINA_SHORT", "Stamina" },
	strength = { "ITEM_MOD_STRENGTH_SHORT", "Strength" },
	agility = { "ITEM_MOD_AGILITY_SHORT", "Agility" },
	intellect = { "ITEM_MOD_INTELLECT_SHORT", "Intellect"},
	spirit = { "ITEM_MOD_SPIRIT_SHORT", "Spirit" },
	
	-- Secondary
	haste = { "ITEM_MOD_HASTE_RATING_SHORT", "Haste Rating" },
	crit = { "ITEM_MOD_CRIT_RATING_SHORT", "Critical Strike Rating"},
	hit = { "ITEM_MOD_HIT_RATING_SHORT", "Hit Rating"},
	resil = { "ITEM_MOD_RESILIENCE_RATING_SHORT", "Resilience"},
	mastery = { "ITEM_MOD_MASTERY_RATING_SHORT", "Mastery Rating"}
}


local itemstruct = {
	["name"] 		= "",
	
	["stamina"] 	= 0,
	["strength"] 	= 0,
	["agility"]		= 0,
	["intellect"] 	= 0,
	["spirit"] 		= 0,
	
	["haste"]		= 0,
	["crit"] 		= 0,
	["hit"] 		= 0,
	["resil"]		= 0,
	["mastery"] 	= 0,
	
	["spell"] 		= 0
}

local CHARSHEET = {
	"Head", "Neck", "Shoulder", "Back", "Chest", "Shirt", "Tabard", "Wrist",
	"Hands", "Waist", "Legs", "Feet", "Finger0", "Finger1", "Trinket0", "Trinket1",
	"MainHand", "OffHand", "Ranged",
}



local defaults = {
	profile = {
		someString = "hello",
		someValue = false
	},
}
	

function Forgery:OnInitialize()
	-- db stuff
	self.db = LibStub("AceDB-3.0"):New("ForgeryDB", defaults, "Default")
		
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Forgery", options, {"Forgery", "ff"})
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Forgery", "Forgery")
	self:RegisterChatCommand("ff", "ChatCommand")
	self:RegisterChatCommand("forgery", "ChatCommand")
end

function Forgery:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(Forgery, "ff", "Forgery", input)
	end
end


function Forgery:OnEnable()
    self:Print("Forgery enabled. Experimental.")
end

function Forgery:OnDisable()
	self:Print("Forgery disabled. Type \"/kc enable\" to enable.")
end


function Forgery:dump_all_stats(stats)
	local i, j, k
	for i in pairs(_STATLIST) do
		if stats[_STATLIST[i][1]] ~= nil then
			self:Print(stats[_STATLIST[i][1]] .. " " .. _STATLIST[i][2])
		end
	end
end

function Forgery:testtable(stats)
	local i, j, k
	for i in pairs(_STATLIST) do
		
	end
end

--
--  Creates a new table based on dummy struct
----------------------------------------------
function Forgery:getNewItemStruct(slotname) 
	local a = {}
	for k,v in pairs(itemstruct) do 
		a[k] = v
	end
	a.name = slotname
	return a
end
--
--  Takes all the values from an item and puts them into a struct
--------------------------------------------------------------------------------
function Forgery:inspectItem(slot, stats)
	local a = self:getNewItemStruct(CHARSHEET[slot])
	local i
	
	for i in pairs(_STATLIST) do
		if stats[_STATLIST[i][1]] ~= nil then
			a[i] = stats[_STATLIST[i][1]]
			--self:Print(i)
		end
	end
	
	return a
end

--
--  Scan character inventory, and for each item, create a structure storing its 
--  combat ratings
--------------------------------------------------------------------------------
function Forgery:processItems()
	local gear = {}
	local i
	for i=1,17 do
		local _item = GetInventoryItemLink("player", i)
		if _item ~= nil then 
			local _stats = GetItemStats(_item)
			table.insert(gear, self:inspectItem(i, _stats))
		end
	end
	return gear
end

-- Display stats of an item
-- (helm)
function Forgery:test()

	--local _item = GetInventoryItemLink("player", 1);
	--self:Print(_item);
--	local _stats = {}
--	GetItemStats(_item, _stats)
	--self:Print(table.getn(_STATLIST))
--	self:Print(_STATLIST["intellect"][1])
	--self:testtable()
--	self:dump_all_stats(_stats)
	--local helm = self:getNewItemStruct("helm")
	--self:Print(helm.name)
	
	-- ok so we can make a dummy struct
	-- and we can iterate through lists
	
	-- can we set dump all the values of helm into struct?
	--local item_one = self:inspectItem(1, _stats)
	--self:Print(item_one.name .. item_one.haste)
	
	-- now let's get all items
	local i
	local gear = self:processItems()
	
	local sum_haste = 0
	for i in pairs(gear) do 
		if (gear[i].haste > 0) then 
			-- we need reforgeable amounts:
			local total = gear[i].haste
			local budget = math.floor(gear[i].haste * 0.40)
			self:Print(gear[i].haste)
			self:Print("Reforgeable: " .. tostring(budget))
			sum_haste = sum_haste + gear[i].haste
		end
	end
	self:Print("Haste total: " .. sum_haste)
	

	
end




-- Saving functions
function Forgery:GetSave(info)
	return self.save
end

function Forgery:SetSave(info, profileName)
    if not profileName or profileName:trim() == "" then
        --
    else
		self:Print("Saving profile: ", profileName)
	end
end

-- Restoring functions
function Forgery:GetRestore(info)
	return self.restore
end

function Forgery:SetRestore(info, profileName)
    if not profileName or profileName:trim() == "" then
        --
    else
       self:Print("restoring profile", profileName)
    end

end

function Forgery:GetRestore(info)
	self:Print("getrestore")
end


function Forgery:Save()
	self:Print("Saving...")
end

function Forgery:Restore()
	self:Print("Restoring...")
end